function Dsq = TDsq_matrix(samples1,samples2,params)
%function Dsq = TDsq_matrix(samples1,samples2,params)
%
% function computing the squared tangent distance matrix between
% all columns of samples1, samples2. If samples2 is empty, then
% samples2==samples1 is assumed. Symmetry will then be used for
% acceleration in the symmetric distance case.
%
% required fields of params:
% datasetname : name of dataset, where samples stem from
% regpar      : regularization parameter used for weighted TD 
%               0 -> ordinary tangent distance, 
%               Inf -> Euclidean distance
% TDtype      : type of TD: 
%                '2S': Two-sided tangent distance
%                '1S': One sided tangent distance
%                'MN': Mean tangent distance
%                'MP': Midpoint tangent distance
%               if not using any tangents, all of these recover the
%               euclidean distance
% normalize   : if this flag is set, normalization of the
%               tangents is performed
% pre_comp_tang2 : if this flag is set, the tangents for samples2
%                  are precomputed, instead of recomputing them
%                  number-of-samples1 times.
%
% plus further parameters as used by compute_tangents:
% tangent_names, tangent_modes, tangent_scale, phi, etc.

% Bernard Haasdonk 22.7.2004

if isempty(samples2) % samples2==samples1
  jmin= (1:size(samples1,2));
  s2 = size(samples1,2);
else % samples2 not equal samples1
  s2 = size(samples2,2);    
  jmin = ones(size(samples1,2),1);
end;

Dsq = zeros(size(samples1,2),s2);

% precompute unnormalized tangents if required
if ismember(params.TDtype,{'MN','2S'})
  if params.pre_comp_tang2
    tang2={};
    for j= 1:s2;
      if isempty(samples2)
	tang2{j} = compute_tangents(samples1(:,j),params);	
      else
	tang2{j} = compute_tangents(samples2(:,j),params);
      end;
    end;
  end; 
end;
  
switch params.TDtype
  %------------
 case '1S'
  for i=1:size(samples1,2);
    t1 = compute_tangents(samples1(:,i),params);
    if params.normalize
      tango1 = orthonormalize(t1);
    else
      tango1 = t1;
    end;
    for j= 1:s2;
      if isempty(samples2)
	Dsq(i,j) = compute_TD1sq(samples1(:,i),samples1(:,j), ...
				 tango1,params);
      else
	Dsq(i,j) = compute_TD1sq(samples1(:,i),samples2(:,j), ...
				 tango1,params);
      end;
    end;    
  end;
 %------------
 case '2S'
  for i=1:size(samples1,2);
    t1 = compute_tangents(samples1(:,i),params);
    for j= jmin(i):s2;
      
      if isempty(samples2)
	sa2 =  samples1(:,j); 
      else
	sa2 =  samples2(:,j);
      end;
      
      if ~params.pre_comp_tang2 	
	t2 = compute_tangents(sa2,params);
      else
	t2 = tang2{j};
      end;
      
      tang = [t1,t2];
      if params.normalize
	tango = orthonormalize(tang);
      else
	tango = tang;
      end;
      Dsq(i,j) = compute_TD1sq(samples1(:,i),sa2,tango, ...
			       params);
    end;
  end;  
  %------------
 case 'MN'    % d_MN = sqrt(0.5*(d1^2 + d2^2))
  for i=1:size(samples1,2);
    t1 = compute_tangents(samples1(:,i),params);
    if params.normalize
      tango1 = orthonormalize(t1);
    else
      tango1 = t1;
    end;
    for j= jmin(i):s2;
      if isempty(samples2)
	sa2 =  samples1(:,j); 
      else
	sa2 =  samples2(:,j);
      end;
      
      if ~params.pre_comp_tang2 
	t2 = compute_tangents(sa2,params);
      else
	t2=tang2{j};
      end;
      
      if params.normalize
	tango2 = orthonormalize(t2);
      else
	tango2 = t2;
      end;
      D1 = compute_TD1sq(samples1(:,i),sa2,tango1,params);
      D2 = compute_TD1sq(sa2,samples1(:,i),tango2,params);
      Dsq(i,j) = 0.5*(D1+D2);
    end;
  end;
  %------------
 case 'MP'
  for i=1:size(samples1,2);
    for j= jmin(i):s2;
      if isempty(samples2)
	sa2 =  samples1(:,j); 
      else
	sa2 =  samples2(:,j);
      end;
      mp = 0.5*(samples1(:,i)+sa2);
      t1 = compute_tangents(mp,params);
      if params.normalize
	tango1 = orthonormalize(t1);
      else
	tango1 = t1;	
      end;
      D1 = compute_TD1sq(mp,sa2,tango1,params);
      D2 = compute_TD1sq(mp,samples1(:,i),tango1,params);
      Dsq(i,j) = ((sqrt(D1)+sqrt(D2)))^2;
    end;
  end;
 otherwise
  error('TDtype unknown!');
end;

% symmetry by copying upper triangle matrix
if isempty(samples2) & ...
      ismember(params.TDtype,{'2S','MN','MP'}) 
  Dsq = Dsq + Dsq' - diag(diag(Dsq));
end;
