function dsq = compute_TD1sq(x1, x2, tang, params)
% function dsq = compute_TD1sq(x1, x2, tang, params)
%
% function computing the squared (one sided) tangent distance between
% x1 and x2 with tangents tang. Hereby tang is assumed to be a matrix
% of tangents as columns. Two-sided TD can be computed by 
% collecting both sets of tangents in tang.
%
% Computation is performed by minimizing:  
%
%         J(alpha) = |\sum alpha_i t_i +x -x'|^2 +regpar sum alpha_i^2
%
% 1. which is in case of orthonormalized t_i equivalent to 
%
%         J(alpha) = (1+regpar) alpha^T alpha +2 M alpha +|x-x'|^2
%
%         with M := (x-x')^T [t_1,...,t_l].
%
%    minimum is obtained by  alpha = -M^T/(1+regpar) 
%
% 2. In the general unnormalized case,
%
%    define  T := [t1,...,t_l]
%
%    and the solution is given by
%
%          alpha = inv(T^T * T + regpar * Id) * T^T * (x-x')
%
%
% required fields of params:
%    regpar    : regularization parameter used for weighted TD 
%                0 -> ordinary tangent distance, 
%                Inf -> Euclidean distance
%    normalize : if this flag is set, normalization of the
%                tangents is assumed.

% Bernard Haasdonk 22.7.2004

%check_required_fields ommitted for speed...

orgdiff = x2-x1;
  
if params.normalize

  if params.regpar==0
    tmp = orgdiff;
    
    for i=1:size(tang,2);
      tmp = tmp - sum(sum((tang(:,i).*orgdiff)))*tang(:,i);
    end;
       
    dsq = sum(sum(tmp.*tmp));
    
  else % compute regularized distance
    M =  -orgdiff' * tang;
    alpha = -1/(1+params.regpar) * M';
    dsq = (1+params.regpar)*alpha' * alpha +2*M*alpha + sum(orgdiff.^2);    
  end;

else % non normalized tangents!!
  alpha = pinv(tang' * tang + params.regpar * eye(size(tang,2))) * tang' ...
	  * orgdiff;
  diff = tang * alpha -orgdiff;
  dsq = diff' * diff + params.regpar*alpha' * alpha;
  dsq = max(0,dsq);
end;

