function tangents = compute_tangents(sample,params)
%function tangents = compute_tangents(sample,params)
%
% function computing unnormalized tangents of a sample into the
% columns of the matrix tangents.
%
% required fields of params:
%    datasetname   : name of dataset for determining the meaning of
%                    given transformations
%    tangent_names : cell array of parameter names for which tangents are to 
%                    be determined 'dx','dy','phi','scale','sigma',...
%                    for each set parameter, the corresponding field must
%                    exist and be set in params if 'f','b', or 'c'
%                    mode is used.
%    tangent_modes : cell array of modes for computation of the tangents:
%                    'f': forward difference, 'b': backward difference
%                    'c': central difference, 'e': exact/explicit derivative
%
% optional fields of params:
%    tangent_scale  : if this vector of real values is given, scaling
%                     of all tangents with the corresponding entry is
%                     performed before normalization.
%    dx 
%    dy
%    phi
%    sigma
%    scale
%    ...

% Bernard Haasdonk 29.7.2004

sample = sample(:);
tangents = zeros(length(sample),0);

tang = {};
lbas = {};

for ti = 1:length(params.tangent_names)
  params.transform_names = params.tangent_names(ti);
  switch params.tangent_modes{ti}
   case 'f'
    tsa = transform_vectors(sample,params);
    ta = tsa-sample;     
   case 'b'
    oldpar = getfield(params,params.tangent_names{ti});
    params = setfield(params,params.tangent_names{ti},-oldpar);
    tsa = transform_vectors(sample,params);
    ta = sample-tsa;      
    params = setfield(params,params.tangent_names{ti},oldpar);
   case 'c'
    oldpar = getfield(params,params.tangent_names{ti});
    params = setfield(params,params.tangent_names{ti},oldpar*0.5);
    tsa1 = transform_vectors(sample,params);
    params = setfield(params,params.tangent_names{ti},-oldpar*0.5);
    tsa2 = transform_vectors(sample,params);
    ta = tsa1-tsa2;      
    params = setfield(params,params.tangent_names{ti},oldpar);
   case 'e'
    switch params.datasetname
      % E.g. USPS explicit tangents, etc.
     otherwise
      error(['exact tangents not implemented for dataset ',...
	     params.datasetname]);
    end;
   otherwise 
    error('tangent-mode not supported.');
  end;
  
  tangents = [tangents, ta(:)];
end;

% perform scaling of the tangents
if isfield(params,'tangent_scale')
  for i = 1:length(params.tangent_scale)
    tangents(:,i) = tangents(:,i)*params.tangent_scale(i);
  end;  
end;


