function pred = predict_kernel_nearest_neighbour(kernel_data,tparams)
%function pred = predict_kernel_nearest_neighbour(kernel_data,tparams)
% 
% function performing kernel nearest neighbour classification on
% certain kernel data.
%
% required fields of kernel_data:
%    Ktraintraindiag : diagonal vector of training kernel matrix
%    Ktraintest      : kernel matrix of train versus test samples.
%    Ktesttestdiag   : diagonal of test kernel matrix
%    labels          : vector of binary labels +1,-1 of the training samples
%
% required fields of tparams:
%    nneigh          : number of neighbours to be used
%
% resulting fields in pred:
%    certainty =     : prediction certainty (ratio of label frequency
%                      in k-nearest-neighbour set)
%    labels          : predicted labels of test samples

% Bernard Haasdonk 25.7.2005

pred = [];
pred.certainty = [];
pred.labels = [];

if length(kernel_data.labels)==1
  disp('nneigh-classification not defined for one training sample!');
  pred.certainty = zeros(1,length(kernel_data.Ktesttestdiag));
  pred.labels = kernel_data.labels*ones(1,length(kernel_data.Ktesttestdiag));
  return;
end;
if tparams.nneigh<=0
  disp('nneigh not positive, taking as 1!');
  tparams.nneigh = 1;
end;
if tparams.nneigh>(length(kernel_data.labels)-1);
  tparams.nneigh = length(kernel_data.labels)-1;
  disp(['nneigh larger than sample number-1 = ',num2str(tparams.nneigh),...
	', taking this bound!']);
end;
if (tparams.nneigh-floor(tparams.nneigh))~=0
  tparams.nneigh = ceil(tparams.nneigh);
  disp(['nneigh not integer, rounding up to ',num2str(tparams.nneigh)]);
end;
Dsqr = repmat(kernel_data.Ktraintraindiag(:),1,...
	      length(kernel_data.Ktesttestdiag))+...
       repmat(kernel_data.Ktesttestdiag(:)',...
	      length(kernel_data.Ktraintraindiag),1) + ...
       -2*kernel_data.Ktraintest;
[sDsqr,i] = sort(Dsqr,1);
if length(kernel_data.labels)==1
  i = i(:)';
end;
sDsqr_topk = sDsqr(1:tparams.nneigh,:);
labels_topk = kernel_data.labels(i(1:tparams.nneigh,:));
if tparams.nneigh > 1
  pred.labels = sign(sum(labels_topk));
else
  pred.labels = labels_topk;
end;
% the certainty is ratio of classes in neighbour set.
pred.certainty = sum(labels_topk==...
		 repmat(pred.labels,tparams.nneigh,1),1)/tparams.nneigh; 