function tparams = train_nu_hypersphere(kernel_data,params)
%function tparams = train_nu_hypersphere(kernel_data,params)
% 
% function performing one-class enclosing hypersphere training 
% with certain kernel training data. To be used for novelty detection.
%
% required fields of kernel_data:
%    Ktraintrain : square, symmetric training kernel matrix
%
% required fields of params:
%    nu       : parameter nu controlling the number of SV in solution
%
% optional fields of params:
%    epsilon : threshold for coefficients to be counted as SV,
%              default 1e-5
%
% resulting fields in tparams:
%    alpha       : vector of solution coefficients 
%    bias        : offset of linear discrimination function

% Klaus Peschke and Bernard Haasdonk 25.7.2005

tparams.alpha = 0;
tparams.bias = 0;

if ~isfield(params,'epsilon')
  params.epsilon = 1e-5;
end;

kdim = size(kernel_data.Ktraintrain,1);
if params.nu>1
  disp('nu larger than one, taking as 1!');
  params.nu = 1;
end;
if params.nu<1/kdim;
  params.nu = 1/kdim;
  disp(['nu smaller than 1/kdim, taking as 1/',num2str(kdim)]);
end;
f = -diag(kernel_data.Ktraintrain);
lb = zeros( kdim,1);
ub =( ones( kdim,1)/( params.nu*kdim));
Aeq = ones( 1,kdim);
beq = 1;
X0 = zeros(1,size(kernel_data.Ktraintrain,1));
op = optimset('Diagnostics','off','LargeScale','off', ...
	      'Display','off');
H = 2*kernel_data.Ktraintrain;
H = 0.5*(H+H');

tparams.alpha = quadprog( H, f,[],[],Aeq,beq,lb,ub,X0,op);
idx = find( (tparams.alpha < 1/(params.nu * kdim)-params.epsilon) & ...
	    (tparams.alpha > params.epsilon) );
if isempty(idx) 
    disp('no unbounded SVs! setting boundary on closest SV');
    i = find(tparams.alpha > params.epsilon); % find all SVs   
    predv = -(diag(kernel_data.Ktraintrain) - ...
	      2* kernel_data.Ktraintrain'*tparams.alpha + ...
	      tparams.alpha'*kernel_data.Ktraintrain* ...
	      tparams.alpha);
    [m,idx] = max(predv(i));
    idx = i(idx);
end;    
idx = idx(1);
% bias corresponds to -rstar2
tparams.bias = -(kernel_data.Ktraintrain(idx,idx) - ...
		 2*( kernel_data.Ktraintrain(idx,:) * tparams.alpha) + ...
		 tparams.alpha'*kernel_data.Ktraintrain*tparams.alpha);