function tparams = train_two_class_C_SVM(kernel_data,params)
%function tparams = train_two_class_C_SVM(kernel_data,params)
%
% function performing two-class C-SVM training with certain kernel
% training data. 
%
% required fields of kernel_data:
%    Ktraintrain : square, symmetric training kernel matrix
%    labels      : vector of binary labels +1,-1 of the training samples
%
% required fields of params:
%    C       : penalization constant in binary SVM
%
% optional fields of params:
%    epsilon : threshold for coefficients to be counted as SV,
%              default 1e-5
%
% resulting fields in tparams:
%    alpha       : vector of solution coefficients 
%    bias        : offset of linear discrimination function

% Bernard Haasdonk 25.7.2005

tparams.alpha = zeros(size(kernel_data.labels));
tparams.bias = 0;
if ~isfield(params,'epsilon')
  params.epsilon = 1e-5;
end;

if params.C<0
  disp('penalization constant C is negative, taking as 0!');
  params.C = 0;
end;
LB = zeros(1,size(kernel_data.Ktraintrain,1));
UB = ones(1,size(kernel_data.Ktraintrain,1))*params.C;
if params.C==Inf
  disp('penalization constant C is infinite, taking as 1e10!');
  UB = 1e10*ones(1,size(kernel_data.Ktraintrain,1));
end;
if size(kernel_data.Ktraintrain,2)==1
  disp('One sample is too few for SVM training!');
  return;
end;
if ismember(length(find(kernel_data.labels==1)),...
	    [0,length(kernel_data.labels)])
  disp('SVM training not possible for only points of one class, aborting!');
  return;
end;
H = diag(kernel_data.labels)* ...
    kernel_data.Ktraintrain*diag(kernel_data.labels);
H = 0.5*(H+H');
f = -ones(1,size(kernel_data.Ktraintrain,1));
A = [];
b = [];
Aeq = kernel_data.labels(:)';
beq = 0;
X0 = zeros(1,size(kernel_data.Ktraintrain,1));
op = optimset('Diagnostics','off','LargeScale','off', ...
	      'Display','off');
%  op = optimset('Diagnostics','off','LargeScale','off');
tparams.alpha = quadprog(H,f,A,b,Aeq,beq,LB,UB,X0,op);
trainclass = sum(diag(kernel_data.labels(:).*tparams.alpha)*...
		 kernel_data.Ktraintrain,1);
iplus = find(kernel_data.labels(:)==1 & tparams.alpha > params.epsilon);
bplus = max(trainclass(iplus)); 
iminus = find(kernel_data.labels(:)==-1 & tparams.alpha > params.epsilon);
bminus = min(trainclass(iminus));
tparams.bias = -0.5*(bminus+bplus);