function tsamples = transform_vectors(samples,params)
%function tsamples = transform_vectors(samples,params)
%
% function performing certain transformations on all columns of
% matrix samples into matrix tsamples.
%
% required fields of params:
%    datasetname     : name of dataset determining the role of the
%                      transformations
%    transform_names : cell array of parameter names for which 
%                      the transformations have to be performed.
%                      'dx','dy','phi','fscale','sigma',...
%                      for each given parameter, the corresponding
%                      params-field as listed below under 'optional' must
%                      exist and be set.
%                      the transformations are performed in the
%                      order as given in this cell array.
%                      if the parameter-fields are vectors, then one
%                      vector is transformed into multiple targets, each
%                      with one parameter combination.
%
% optional fields of params (scalar or vectorial):
%    dx       
%    sigma
%    fscale
%    df
%    dy
%    phi
%    scale
%    eps    
%    ...
%    center      : in case of datasetname '2D' data 
%                  the center for scaling and rotation

% Bernard Haasdonk 29.7.2004

tsamples = samples;

for ti = 1:length(params.transform_names);
  nsamples = {};
  npars = length(getfield(params,params.transform_names{ti}));
  for pars =1:npars
    nsamples{pars} = tsamples;
    switch params.datasetname
      %%%%%%%%%%%%%%%%%%%%%%%% 2D data %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
     case '2D'
      switch params.transform_names{ti}
	% params.dx
       case 'dx'
	nsamples{pars}(1,:)  = tsamples(1,:) + params.dx(pars);
       case 'dy'
	nsamples{pars}(2,:)  = tsamples(2,:) + params.dy(pars);
       case 'dxy'
	nsamples{pars} = tsamples + params.dxy(pars) * ... 
                         repmat([1; 0.5],1,size(tsamples,2));
       case 'phi'
	cp = cos(params.phi(pars));
	sp = sin(params.phi(pars));
	rotp = repmat(params.center,1,size(tsamples,2));
	nsamples{pars}  = [cp, -sp; ...
		     sp, cp] * (tsamples-rotp)+rotp;
       case 'scale'      
	rotp = repmat(params.center,1,size(tsamples,2));
	nsamples{pars}  = (tsamples-rotp) * exp(params.scale(pars))+rotp;
       case 'dphi' % shift along a sine curve
		   % for a single point:
		   % dy = y - sin(10*x);
		   % x = x + dphi
		   % y = sin(10*x)+dy;
		   amp = 0.5;
		   freq = 14;
		   dy = tsamples(2,:) - amp*sin(freq*tsamples(1,:));
		   nsamples{pars}(1,:) = tsamples(1,:)+ params.dphi(pars);
		   nsamples{pars}(2,:) = amp*sin(freq*nsamples{pars}(1,:)) + dy;
       case 'eps' % reflection along perpendicular line through center of trafo
                  % parameter only 0 and 1 allowed
		  if params.eps(pars)==1
		    nsamples{pars}(1,:) = 2*params.center(1)-tsamples(1,:);
		  end;
       otherwise
	error(['transformations not implemented for dataset ',...
	       params.datasetname]);
      end;   
      
      %%%%%%%%%%%%%%%%%%%%%%%% future data %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
      %  case 'some-future-dataset'
      %   switch params.transform_names{ti}
      %    case 'scale' % uniform scaling of the function values
      %    otherwise
      %     error(['transformation not implemented for dataset ',...
      %           params.datasetname]);
      %    end;   
      
     otherwise
      error(['transformations not implemented for dataset ',params.datasetname]);
    end; 
  end; % pars
  % merge all results into tsamples and resort
  tsamples = [nsamples{:}];
  nsamp = size(tsamples,2)/npars;
  v = reshape(1:(npars*nsamp),nsamp,npars)';
  v = v(:);
  tsamples = tsamples(:,v);
end;







