function [K,ntrans,P] = virtual_samples_kernel_matrix(samples1,samples2, ...
			       params);
% function [K,ntrans,P] = virtual_samples_kernel_matrix(samples1,samples2, ...
%				 params);
%
% function computing a block-kernel-matrix K between the
% (column)-vectors in samples1 and samples2 and all of their
% transformed samples, i.e. the kernel matrix of all "virtual"
% samples. ntrans is the number of transformed patterns per input
% sample. 
% 
% If samples2 is empty, then samples2==samples1 is assumed. 
% Symmetry can then be used for
% acceleration in symmetric distance case.
% ntrans is the size of the subblocks, the number of transformed
% patterns of each sample.
%
% P is a length(transform_names) times ntrans matrix of
% transformation parameters. Each subblock of K is corresponding to 
% those parameter values.
%
% required fields of params:
% kernel_matrix_func : name of the function for computation of the base-kernel
% transform_names    : cell array of parameter names for which the 
%                      transformations have to be performed.
%                      'dx','dy','phi','fscale','sigma',...
% transform_nfeval   : array of (odd) integers indicating the
%                      number of equidistant function evaluations 
%                      to be used for each parameter in 
%                      transform_names
% transform_min      : array of values indicating the lower
%                      parameter boundary to be used for each
%                      parameter in transform_names
% transform_max      : array of values indicating the upper
%                      parameter boundary to be used for each 
%                      parameter in in transform_names
%
% additional parameters as used by transform_vectors and
% params.kernel_matrix_func must be set.

% Bernard Haasdonk 7.6.2005

if isempty(samples2) % samples2==samples1
  s2 = size(samples1,2);
else % samples2 not equal samples1
  s2 = size(samples2,2);    
end;

% set transformation-value vectors and determine ntrans
ntrans = 1;
P = []; % ntrans x length(transform_names) array of trafo-parameters
for ti = 1:length(getfield(params,'transform_names'));
  par = getfield(params,params.transform_names{ti});
  parv = 0:(params.transform_nfeval(ti)-1);
  if length(parv)>1
    parv = parv/(length(parv)-1)*...
	   (params.transform_max(ti)-params.transform_min(ti));
  end;
  parv = parv+params.transform_min(ti);
  params = setfield(params,params.transform_names{ti},parv);
  % stretch P to new length by factor length(parv)
  P = reshape(repmat(P,length(parv),1),(ti-1),ntrans*length(parv));
  % add new parameter vector
  P = [P; repmat(parv,1,ntrans)];
  ntrans = ntrans * length(parv);
end;

tsamples1 = transform_vectors(samples1,params);
if ~isempty(samples2) 
  tsamples2 = transform_vectors(samples2,params);
else 
  tsamples2 = [];
end;

K = eval([params.kernel_matrix_func,'(tsamples1,tsamples2,params);']);

% check whether transformed samples indeed are produced by
% extracted parameters in P
%for i = 1:ntrans
%  for j = 1:length(params.transform_names);
%    params = setfield(params,params.transform_names{j},P(j,i));
%  end;
%  tsample = transform_vectors(samples1(:,1),params);
%  if ~isequal(tsample, tsamples1(:,i))
%    disp('error in computation of parameter matrix P!');
%    keyboard;
%  end;
%end;
%disp('OK, computation of P is fine.');
%keyboard;
